@@ -347,6 +347,10 @@ private:
 
 int ComplexKafkaTask::get_node_id(const KafkaToppar *toppar)
 {
+	int preferred_read_replica = toppar->get_preferred_read_replica();
+	if (preferred_read_replica >= 0)
+		return preferred_read_replica;
+
 	bool flag = false;
 	this->client_meta_list.rewind();
 	KafkaMeta *meta;
@@ -417,8 +421,8 @@ void ComplexKafkaTask::kafka_rebalance_callback(__WFKafkaTask *task)
 
 			kafka_task = __WFKafkaTaskFactory::create_kafka_task(addr,
 																 socklen,
-																 0,
 																 t->get_userinfo(),
+																 0,
 																 kafka_heartbeat_callback);
 			kafka_task->user_data = t;
 			kafka_task->get_req()->set_api_type(Kafka_Heartbeat);
@@ -521,8 +525,8 @@ void ComplexKafkaTask::kafka_timer_callback(WFTimerTask *task)
 	socklen_t socklen;
 	coordinator->get_broker_addr(&addr, &socklen);
 
-	kafka_task = __WFKafkaTaskFactory::create_kafka_task(addr, socklen, 0,
-														 t->get_userinfo(),
+	kafka_task = __WFKafkaTaskFactory::create_kafka_task(addr, socklen,
+														 t->get_userinfo(), 0,
 														 kafka_heartbeat_callback);
 
 	kafka_task->user_data = t;
@@ -629,8 +633,8 @@ void ComplexKafkaTask::kafka_cgroup_callback(__WFKafkaTask *task)
 			coordinator->get_broker_addr(&addr, &socklen);
 
 			kafka_task = __WFKafkaTaskFactory::create_kafka_task(addr, socklen,
-																 t->retry_max,
 																 t->get_userinfo(),
+																 t->retry_max,
 																 kafka_heartbeat_callback);
 			kafka_task->user_data = hb;
 			kafka_task->get_req()->set_config(t->config);
@@ -889,16 +893,16 @@ void ComplexKafkaTask::dispatch()
 			broker->get_broker_addr(&addr, &socklen);
 
 			task = __WFKafkaTaskFactory::create_kafka_task(addr, socklen,
-														   this->retry_max,
 														   this->get_userinfo(),
+														   this->retry_max,
 														   kafka_cgroup_callback);
 		}
 		else
 		{
 			task = __WFKafkaTaskFactory::create_kafka_task(broker->get_host(),
 														   broker->get_port(),
-														   this->retry_max,
 														   this->get_userinfo(),
+														   this->retry_max,
 														   kafka_cgroup_callback);
 		}
 
@@ -942,16 +946,16 @@ void ComplexKafkaTask::dispatch()
 				broker->get_broker_addr(&addr, &socklen);
 
 				task = __WFKafkaTaskFactory::create_kafka_task(addr, socklen,
-															   this->retry_max,
 															   this->get_userinfo(),
+															   this->retry_max,
 															   nullptr);
 			}
 			else
 			{
 				task = __WFKafkaTaskFactory::create_kafka_task(broker->get_host(),
 															   broker->get_port(),
-															   this->retry_max,
 															   this->get_userinfo(),
+															   this->retry_max,
 															   nullptr);
 			}
 
@@ -994,16 +998,16 @@ void ComplexKafkaTask::dispatch()
 				broker->get_broker_addr(&addr, &socklen);
 
 				task = __WFKafkaTaskFactory::create_kafka_task(addr, socklen,
-															   this->retry_max,
 															   this->get_userinfo(),
+															   this->retry_max,
 															   nullptr);
 			}
 			else
 			{
 				task = __WFKafkaTaskFactory::create_kafka_task(broker->get_host(),
 															   broker->get_port(),
-															   this->retry_max,
 															   this->get_userinfo(),
+															   this->retry_max,
 															   nullptr);
 			}
 
@@ -1044,8 +1048,8 @@ void ComplexKafkaTask::dispatch()
 			coordinator->get_broker_addr(&addr, &socklen);
 
 			task = __WFKafkaTaskFactory::create_kafka_task(addr, socklen,
-														   this->retry_max,
 														   this->get_userinfo(),
+														   this->retry_max,
 														   kafka_offsetcommit_callback);
 			task->user_data = this;
 			task->get_req()->set_config(this->config);
@@ -1078,8 +1082,8 @@ void ComplexKafkaTask::dispatch()
 			{
 				task = __WFKafkaTaskFactory::create_kafka_task(addr,
 															   socklen,
-															   0,
 															   this->get_userinfo(),
+															   0,
 															   kafka_leavegroup_callback);
 				task->user_data = this;
 				task->get_req()->set_config(this->config);
