@@ -239,6 +239,17 @@ int CommService::drain(int max)
 	return cnt;
 }
 
+inline void CommService::incref()
+{
+	__sync_add_and_fetch(&this->ref, 1);
+}
+
+inline void CommService::decref()
+{
+	if (__sync_sub_and_fetch(&this->ref, 1) == 0)
+		this->handle_unbound();
+}
+
 class CommServiceTarget : public CommTarget
 {
 public:
@@ -305,7 +316,7 @@ CommSession::~CommSession()
 
 inline int Communicator::first_timeout(CommSession *session)
 {
-	int timeout = session->response_timeout();
+	int timeout = session->target->response_timeout;
 
 	if (timeout < 0 || (unsigned int)session->timeout <= (unsigned int)timeout)
 	{
@@ -321,7 +332,7 @@ inline int Communicator::first_timeout(CommSession *session)
 
 int Communicator::next_timeout(CommSession *session)
 {
-	int timeout = session->response_timeout();
+	int timeout = session->target->response_timeout;
 	struct timespec cur_time;
 	int time_used, time_left;
 
@@ -1520,7 +1531,7 @@ int Communicator::request(CommSession *session, CommTarget *target)
 			data.fd = entry->sockfd;
 			data.ssl = NULL;
 			data.context = entry;
-			timeout = session->connect_timeout();
+			timeout = session->target->connect_timeout;
 			if (mpoller_add(&data, timeout, this->mpoller) >= 0)
 				break;
 
