@@ -35,6 +35,7 @@
 #include <snappy.h>
 #include <snappy-sinksource.h>
 #include "crc32c.h"
+#include "EncodeStream.h"
 #include "KafkaMessage.h"
 
 namespace protocol
@@ -222,6 +223,17 @@ static inline size_t append_varint_i32(std::string& buf, int32_t num)
 	return append_varint_i64(buf, num);
 }
 
+static size_t append_compact_string(std::string& buf, const char *str)
+{
+	if (!str || str[0] == '\0')
+		append_string(buf, "");
+
+	size_t len = strlen(str);
+	size_t r = append_varint_u64(buf, len + 1);
+	append_string_raw(buf, str, len);
+	return r + len;
+}
+
 static inline int parse_i8(void **buf, size_t *size, int8_t *val)
 {
 	if (*size >= 1)
@@ -2131,8 +2143,12 @@ int KafkaRequest::encode_produce(struct iovec vectors[], int max)
 			KafkaBlock record_block;
 			struct timespec ts;
 
-			clock_gettime(CLOCK_REALTIME, &ts);
-			record->get_raw_ptr()->timestamp = (ts.tv_sec * 1000000000 + ts.tv_nsec) / 1000 / 1000;
+			if (record->get_timestamp() == 0)
+			{
+				clock_gettime(CLOCK_REALTIME, &ts);
+				record->set_timestamp((ts.tv_sec * 1000000000 +
+									   ts.tv_nsec) / 1000 / 1000);
+			}
 
 			if (batch_cnt == 0)
 			{
@@ -2398,7 +2414,12 @@ int KafkaRequest::encode_fetch(struct iovec vectors[], int max)
 
 	//rackid
 	if (this->api_version >= 11)
-		append_string(this->msgbuf, "");
+	{
+		if (this->config.get_rack_id())
+			append_compact_string(this->msgbuf, this->config.get_rack_id());
+		else
+			append_string(this->msgbuf, "");
+	}
 
 	this->cur_size = this->msgbuf.size();
 
@@ -3228,7 +3249,10 @@ int KafkaResponse::parse_fetch(void **buf, size_t *size)
 			}
 
 			if (this->api_version >= 11)
+			{
 				CHECK_RET(parse_i32(buf, size, &preferred_read_replica));
+				ptr->preferred_read_replica = preferred_read_replica;
+			}
 
 			parse_records(buf, size, this->config.get_check_crcs(),
 						  toppar->get_record(),
@@ -3284,18 +3308,14 @@ int KafkaResponse::parse_listoffset(void **buf, size_t *size)
 				if (ptr->offset_timestamp == -1)
 				{
 					CHECK_RET(parse_i64(buf, size, (int64_t *)&ptr->high_watermark));
-					if (ptr->offset > ptr->high_watermark ||
-						this->config.get_offset_timestamp() == -1)
-						ptr->offset = ptr->high_watermark;
+					if (ptr->offset > ptr->high_watermark || ptr->offset == -1)
+						ptr->offset = ptr->high_watermark - 1;
 				}
 				else if (ptr->offset_timestamp == -2)
 				{
 					CHECK_RET(parse_i64(buf, size, (int64_t *)&ptr->low_watermark));
-					if (this->config.get_offset_timestamp() == -2 &&
-						ptr->offset < ptr->low_watermark)
-					{
+					if (ptr->offset < ptr->low_watermark)
 						ptr->offset = ptr->low_watermark;
-					}
 				}
 				else
 				{
